#!/usr/bin/perl -w

# adam mathes 8/9/2004
# this software is licensed under a BSD-style license
# please see LICENSE for restrictions


use Net::OSCAR qw(:standard);
use Config::Natural;
use HTML::Template;
use HTML::Scrubber;
use vars qw(@stalkees %messages %profiles $config 
		$doWeblog $doXML $doRSS $doHTML
		@recentMessages 
		$MAX_MESSAGES
		$OUTPUT_DIRECTORY
		$XML_FILE $XML_TEMPLATE
		$RSS_FILE $RSS_TEMPLATE
		$HTML_FILE $HTML_TEMPLATE
		$TITLE
		$URL
		$DESCRIPTION
		$VERBOSE
		$SCRUBBER
		); 
use strict;

# config file location - you might need to change this
my $configFile = 'awaytoweb.conf';
$config = new Config::Natural $configFile || die 'Couldn\'t open configuration';

$doWeblog = $config->param('doWeblog');
if('$doWeblog') {
	use Net::Blogger;
}

my $screenname = $config->param('screenname');
my $password = $config->param('password');

$VERBOSE = $config->param('verbose'); # for debugging
$OUTPUT_DIRECTORY = $config->param('outputDirectory');

$doXML = $config->param('outputXML');
if($doXML) {
	$XML_FILE = $config->param('xmlFile');
	$XML_TEMPLATE = $config->param('xmlTemplate');
}
$doRSS = $config->param('outputRSS'); 
if($doRSS) {
	$RSS_FILE = $config->param('rssFile');
	$RSS_TEMPLATE = $config->param('rssTemplate');
}
$doHTML = $config->param('outputHTML');
if($doHTML) {
	$HTML_FILE = $config->param('htmlFile');
	$HTML_TEMPLATE = $config->param('htmlTemplate');
}


$MAX_MESSAGES = $config->param('maxMessages');
$TITLE = $config->param('title');
$URL = $config->param('url');
$DESCRIPTION = $config->param('description');

$SCRUBBER = HTML::Scrubber->new( allow => [ qw[ p b i u br ] ] );

# the screennames you want to check up on
@stalkees = GetStalkees();

# initialize the messages and profiles

# turn autoflush on
$| = 1;

# the time, in seconds, between away message checking 
my $frequency = $config->param('frequency');
my $delay = $config->param('delay');


# the logfile to write to
my $logfile = "amstalkerlog";
my $signedon = 0;


my $oscar = Net::OSCAR->new();
$oscar->set_callback_im_in(\&on_im_in);
$oscar->set_callback_buddy_info(\&buddy_info);
$oscar->set_callback_error(\&on_error);
$oscar->set_callback_signon_done(\&signon_done);
$oscar->set_callback_rate_alert(\&on_rate_alert);
$oscar->set_callback_evil(\&evil);

$oscar->signon($screenname, $password);

# wait until we actually sign on
while(1) {
	$oscar->do_one_loop();
	if($oscar->is_on) {
		last;
	}	
}

# initialize some globals and add buddies to the
# buddy list if they aren't there
# we can't check messages if they aren't on our
# list (apparently)
foreach my $stalkee (@stalkees) {
	AddBuddy($stalkee);
	$messages{$stalkee} = '-';
	$profiles{$stalkee} = '-'; 
}
$oscar->commit_buddylist();



# main loop
# we just check and check and check 
while(1) {

	if($VERBOSE) {
		print "checking screennames\n the time is " . localtime() . "\n";
	}
	foreach my $sn (@stalkees) {
		if($VERBOSE) { print "checking $sn\n"; }
		my $buddy = $oscar->buddy($sn);
		$oscar->do_one_loop();

		if($$buddy{'online'}) {
			$oscar->get_away($sn);
			$oscar->do_one_loop();
			$oscar->get_info($sn);
			$oscar->do_one_loop();
			my $slept = sleep($delay);
			if($VERBOSE) { print "slept for $slept seconds\n"; }
		}
		else {
			#print "$sn is not online\n";
		}
	}
	my $time = time();
	if($VERBOSE) { print "waiting $frequency seconds\n"; }
	while(time()-$time < $frequency) {
	    $oscar->do_one_loop();
	}
}



sub evil($$$) {
	shift;
        my($newevil, $enemy) = @_;
        $enemy ||= "Anonymous";
        print "$enemy has just evilled you!  Your new evil level is $newevil%.\n";
	$oscar->add_deny($enemy);
	$oscar->commit_buddylist;
}


sub on_rate_alert($$$) {
        my ($oscar, $level, $clear, $window, $worrisome) = @_;

	# if($VERBOSE) {  print "got a rate_alert... with level $level\n"; }
	if($level == RATE_DISCONNECT) {
		print "uhoh, about to be disconnected";
	}
	if($level == RATE_ALERT) {
	#	print "alert!!!\n";
	#	return;
	}
	if($level == RATE_CLEAR ) {
		# print "we're clear\n";
		return;
	}
	if($level == RATE_LIMIT) {
		#print "rate limited by server!\n";
	}

	if($worrisome) {
	        $clear /= 1000;
		$clear /= $window;
		$clear *= 3;
		if($clear < 2 ) {
			$clear = 2;
		}
	        #print "We received a level $level rate alert.\nWaiting for about $clear seconds\n";
		sleep($clear);
	}
}


sub signon_done {
	my $oscar = shift;
	print "signed on to AIM service\n";
}


sub on_error {
	(my $oscar, my $connection, my $error, my $description, my $fatal) = @_;
	print "got an error: error code $error\n";
	print "description: $description\n";
	if($fatal) {
		die "error is fatal!";
	}
}


sub on_im_in {
	(my $oscar, my $from, my $message, my $away) = @_;
	$message = StripHTML($message);
	print "from: $from\n";
	print "message: $message\n";
}


sub buddy_info($$$$) {
	shift;
	my($screenname, $buddat) = @_;
	my $away = $buddat->{awaymsg};
	my $profile = $buddat->{profile};

	my $now = localtime;
	if($away) {
		$away = CleanMessage($away);
        	if(IsNewMessage($away,$screenname)) {
        	    AddMessage($away,$screenname);
        	}        
	}
	if($profile) {
		$profile = CleanMessage($profile);
        	if(IsNewMessage($profile,$screenname)) {
        	    AddProfile($profile,$screenname);
		    #AddMessage($profile,$screenname);
        	}
	}
}

sub CleanMessage {
	my $s = shift;
       	$s = $SCRUBBER->scrub($s);
	return($s);
}


sub StripHTML {
	my $s = shift;
	$s =~ s/<.+?>//g; 
	return $s;
}

sub IsNewMessage {
    my $away = shift;
    my $screenname = shift;

	if($VERBOSE) {
    print "checking for new message\n";
    print "$messages{$screenname} (old)\n";
    print "$away (new)\n";
	}
    return(!($messages{$screenname} eq $away));
}

sub AddMessage {
    my $away = shift;
    my $screenname = shift;

	if($VERBOSE) {
    	print "new message!\n";
    	print "from: $screenname\n";
	print "$away\n";
	print "------\n";  
	}
    if($doWeblog) {
	PostToWeblog($screenname, $away);
    }

   my %message;
   $message{sn} = $screenname;
   $message{text} = $away;
   $message{datetime} = time();
 
    unshift(@recentMessages, \%message);
    if(@recentMessages > $MAX_MESSAGES) {
	pop(@recentMessages);
    }

    if($doRSS) {
	OutputRSS();
    }

    if($doXML) {
	OutputXML();
    }

    if($doHTML) {
	OutputHTML();	
    }

    $messages{$screenname} = $away;
}


sub AddProfile {
    my $away = shift;
    my $screenname = shift;
}



sub AddBuddy {
    my $screenname = shift;
    my $sql;

    $screenname = normalize($screenname);

    $oscar->add_buddy('buddies',$screenname);
 #   $oscar->commit_buddylist();
    $oscar->do_one_loop();

#    if(UserExists($screenname)) {
#        $screenname = $dbh->quote($screenname);
#        $sql = "update users set status=1 where screenname=$screenname;";
#    }
#    else {
#        $screenname = $dbh->quote($screenname);
#        $sql = "insert into users (screenname,status) values ($screenname,1);";
#    }
#    DoSql($dbh,$sql);
}

sub RemoveBuddy {
    my $screenname = shift;
    my $sql;
    $screenname = normalize($screenname);

	$oscar->remove_buddy('Buddies',$screenname);
	$oscar->commit_buddylist();
	$oscar->do_one_loop();

}


sub normalize {
	my $s = shift;
	$s = lc($s);
	$s =~ s/\W//gis;
	return($s);
}

sub GetStalkees {
	my $file = 'buddies.txt';
	my @stalkees;
	open (INPUT, $file)         || die "can't open $file: $!";
	while (<INPUT>) {
		chomp;
		push @stalkees, $_;
	}
	close(INPUT)                || die "can't close $file: $!";
	return(@stalkees);
}

sub PostToWeblog {
	my $screenname = shift;
	my $message = shift;

	print "posting to site\n";

	my $mt = Net::Blogger->new(engine=>"movabletype");

	my $id = $config->param('bloggerId');
	my $proxy = $config->param('bloggerApiUrl');
	my $user = $config->param('bloggerUsername');
	my $pass = $config->param('bloggerPassword');

        $mt->BlogId($id);
        $mt->Proxy("$proxy");
        $mt->Username("$user");
        $mt->Password("$pass");

	my $postid_1 = $mt->newPost(postbody=>\"$message <p>$screenname</p>",publish=>1) || croak $mt->LastError();
}

sub OutputRSS {
	GeneralOutput($RSS_TEMPLATE, $RSS_FILE);
}

sub OutputXML {
	GeneralOutput($XML_TEMPLATE, $XML_FILE);
}

sub OutputHTML {
	GeneralOutput($HTML_TEMPLATE, $HTML_FILE);
}

sub GeneralOutput {
	my $template = shift;
	my $outputfile = shift;

	my %s;
	$s{title} = $TITLE;
	$s{url} = $URL;
	$s{description} = $DESCRIPTION;
	$s{messages} = \@recentMessages;
	my $output = ParseTemplate($template, %s);
	WriteToFile($output, $outputfile);
}


sub ParseTemplate {
    (my $filename, my %s) = @_;

    my $template = HTML::Template->new(die_on_bad_params=>0, filename => "$filename", loop_context_vars => 1, global_vars=>1);
    $template->param(%s);
    my $output = $template->output;
    $output = ProcessDates($output);
#    $output = ProcessEscapes($output);
    return($output);
}


sub ProcessDates {
  my $s = shift;

  $s =~ s/<time="(.*)" format="(.*)">/POSIX::strftime( "$2", localtime($1))/eg;
  return $s;
}

sub WriteToFile {
  my $output = shift;
  my $filename = shift;

  open(OUTPUT,">$OUTPUT_DIRECTORY/$filename") || die "couldn't open $OUTPUT_DIRECTORY/$filename";
  print OUTPUT $output;
  close(OUTPUT);
}
